@@ -1,73 +1,96 @@
-;;; Mailing, forwarding, and replying commands for VM
-;;; Copyright (C) 1989-2001 Kyle E. Jones
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 1, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;;; vm-reply.el --- Mailing, forwarding, and replying commands
+;;
+;; Copyright (C) 1989-2001 Kyle E. Jones
+;; Copyright (C) 2003-2006 Robert Widhopf-Fenk
+;;
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2 of the License, or
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License along
+;; with this program; if not, write to the Free Software Foundation, Inc.,
+;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-;;(provide 'vm-reply)
 
+;;; Code:
+
+(defun vm-add-reply-subject-prefix (message &optional start)
+  (when (not start)
+    (goto-char (point-min))
+    (re-search-forward (regexp-quote mail-header-separator) (point-max))
+    (forward-char 1)
+    (setq start (point)))
+  (goto-char start)
+  (if (and message vm-included-text-attribution-format)
+      (let ((vm-summary-uninteresting-senders nil))
+        (insert (vm-summary-sprintf
+                 vm-included-text-attribution-format
+                 message))))
+  (while (re-search-forward "^" (point-max) t)
+    (insert vm-included-text-prefix)))
+
+;;;###autoload
 (defun vm-do-reply (to-all include-text count)
     (let ((mlist (vm-select-marked-or-prefixed-messages count))
 	  (dir default-directory)
 	  (message-pointer vm-message-pointer)
 	  (case-fold-search t)
-	  to cc subject mp in-reply-to references tmp tmp2 newsgroups)
+	  to cc subject in-reply-to references
+	  mp tmp tmp2 newsgroups)
       (setq mp mlist)
-      (while mp 
-	(cond
-	 ((eq mlist mp)
-	  (cond ((setq to
+      (while mp
+	(cond ((add-to-list 'to
 		       (let ((reply-to
 			      (vm-get-header-contents (car mp) "Reply-To:"
 						      ", ")))
 			 (if (vm-ignored-reply-to reply-to)
 			     nil
 			   reply-to ))))
-		((setq to (vm-get-header-contents (car mp) "From:" ", ")))
+	      ((add-to-list 'to (vm-get-header-contents (car mp) "From:"
+							", ")))
 		;; bad, but better than nothing for some
-		((setq to (vm-grok-From_-author (car mp))))
+	      ((add-to-list 'to (vm-grok-From_-author (car mp))))
 		(t (error "No From: or Reply-To: header in message")))
-	  (setq subject (vm-get-header-contents (car mp) "Subject:")
-		in-reply-to
-		(and vm-in-reply-to-format
+
+	(let ((this-subject (vm-get-header-contents (car mp) "Subject:"))
+	      (this-reply-to (and vm-in-reply-to-format
 		     (let ((vm-summary-uninteresting-senders nil))
-		       (vm-summary-sprintf vm-in-reply-to-format (car mp))))
-		in-reply-to (and (not (equal "" in-reply-to)) in-reply-to))
-	  (and subject (stringp vm-reply-subject-prefix)
-	       (let ((case-fold-search t))
-		  (not
-		   (equal
-		    (string-match (regexp-quote vm-reply-subject-prefix)
-				  subject)
-		    0)))
-	       (setq subject (concat vm-reply-subject-prefix subject))))
-	 (t (cond ((setq tmp (vm-get-header-contents (car mp) "Reply-To:"
-						     ", "))
-		   (setq to (concat to "," tmp)))
-		  ((setq tmp (vm-get-header-contents (car mp) "From:"
-						     ", "))
-		   (setq to (concat to "," tmp)))
+				    (vm-summary-sprintf vm-in-reply-to-format
+							(car mp))))))
+	  (if (and this-subject vm-reply-subject-prefix
+		   (not (string-match vm-reply-subject-prefix this-subject)))
+              (setq this-subject (concat vm-reply-subject-prefix
+                                         this-subject)))
+	  
+          (unless subject
+            (setq subject (concat this-subject
+                                  (if (cdr mlist)
+                                      (format " [and %d more messages]"
+                                              (length (cdr mlist)))))))
+          (setq in-reply-to (if in-reply-to
+				(concat in-reply-to ",\n\t" this-reply-to)
+			      this-reply-to)))
+		
+	(cond ((setq tmp (vm-get-header-contents (car mp) "Reply-To:" ", "))
+               (if (not (vm-ignored-reply-to tmp))
+                   (add-to-list 'to tmp)))
+	      ((setq tmp (vm-get-header-contents (car mp) "From:" ", "))
+	       (add-to-list 'to tmp))
 		  ;; bad, but better than nothing for some
 		  ((setq tmp (vm-grok-From_-author (car mp)))
-		   (setq to (concat to "," tmp)))
-		  (t (error "No From: or Reply-To: header in message")))))
+	       (add-to-list 'to tmp))
+	      (t (error "No From: or Reply-To: header in message")))
+
 	(if to-all
 	    (progn
-	      (setq tmp (vm-get-header-contents (car mp) "To:"
-						", "))
-	      (setq tmp2 (vm-get-header-contents (car mp) "Cc:"
-						 ", "))
+	      (setq tmp (vm-get-header-contents (car mp) "To:" ", "))
+	      (setq tmp2 (vm-get-header-contents (car mp) "Cc:" ", "))
 	      (if tmp
 		  (if cc
 		      (setq cc (concat cc "," tmp))
@@ -82,10 +105,21 @@
 		    (cons (vm-get-header-contents (car mp) "Message-ID:" " ")
 			  references)))
 	(setq newsgroups
-	      (cons (or (and to-all (vm-get-header-contents (car mp) "Followup-To:" ","))
+	      (cons (or (and to-all
+			     (vm-get-header-contents (car mp)
+						     "Followup-To:" ","))
 			(vm-get-header-contents (car mp) "Newsgroups:" ","))
 		    newsgroups))
 	(setq mp (cdr mp)))
+
+      (if (null to) nil
+	(setq tmp (car to))
+	(setq to (cdr to))
+	(while to
+	  (setq tmp (concat tmp ", " (car to)))
+	  (setq to (cdr to)))
+	(setq to tmp))
+
       (if vm-strip-reply-headers
 	  (let ((mail-use-rfc822 t))
 	    (and to (setq to (mail-strip-quoted-names to)))
@@ -151,15 +185,16 @@
   addresses )
 
 (defun vm-ignored-reply-to (reply-to)
-  (if reply-to
+  (if (and reply-to (not (string= reply-to "")))
       (let (re-list result)
 	(setq re-list vm-reply-ignored-reply-tos)
 	(while re-list
 	  (if (string-match (car re-list) reply-to)
 	      (setq result t re-list nil)
 	    (setq re-list (cdr re-list))))
-	result )))
+	result)))
 
+;;;###autoload
 (defun vm-mail-yank-default (&optional message)
   (save-excursion
     (vm-reorder-message-headers nil vm-included-text-headers
@@ -181,6 +216,7 @@
 	(insert vm-included-text-prefix)
 	(forward-line 1)))))
 
+;;;###autoload
 (defun vm-yank-message-other-folder (folder)
   "Like vm-yank-message except the message is yanked from a folder other
 than the one that spawned the current Mail mode buffer.  The name of the
@@ -217,7 +253,7 @@ Don't call this function from a program."
 	(while (zerop result)
 	  (setq result (read-string prompt))
 	  (and (string= result "") default (setq result default))
-	  (setq result (string-to-int result)))
+	  (setq result (string-to-number result)))
 	(if (null (setq mp (nthcdr (1- result) vm-message-list)))
 	    (error "No such message."))))
     (set-buffer b)
@@ -227,6 +263,7 @@ Don't call this function from a program."
       (vm-bury-buffer newbuf)
       (vm-bury-buffer sumbuf))))
 
+;;;###autoload
 (defun vm-yank-message (message)
   "Yank message number N into the current buffer at point.
 When called interactively N is always read from the minibuffer.  When
@@ -239,26 +276,26 @@ are replying to, forwarding, or invoked VM's mail command from.
 
 All message headers are yanked along with the text.  Point is
 left before the inserted text, the mark after.  Any hook
-functions bound to mail-citation-hook are run, after inserting
+functions bound to `mail-citation-hook' are run, after inserting
 the text and setting point and mark.  For backward compatibility,
 if mail-citation-hook is set to nil, `mail-yank-hooks' is run
 instead.
 
 If mail-citation-hook and mail-yank-hooks are both nil, this
 default action is taken: the yanked headers are trimmed as
-specified by vm-included-text-headers and
-vm-included-text-discard-header-regexp, and the value of
-vm-included-text-prefix is prepended to every yanked line."
+specified by `vm-included-text-headers' and
+`vm-included-text-discard-header-regexp', and the value of
+`vm-included-text-prefix' is prepended to every yanked line."
   (interactive
    (list
-   ;; What we really want for the first argument is a message struct,
-   ;; but if called interactively, we let the user type in a message
-   ;; number instead.
+    ;; What we really want for the first argument is a message struct,
+    ;; but if called interactively, we let the user type in a message
+    ;; number instead.
     (let (mp default
-	  (result 0)
-	  prompt
-	  (last-command last-command)
-	  (this-command this-command))
+             (result 0)
+             prompt
+             (last-command last-command)
+             (this-command this-command))
       (save-excursion
 	(vm-select-folder-buffer)
 	(setq default (and vm-message-pointer
@@ -270,7 +307,7 @@ vm-included-text-prefix is prepended to every yanked line."
 	(while (zerop result)
 	  (setq result (read-string prompt))
 	  (and (string= result "") default (setq result default))
-	  (setq result (string-to-int result)))
+	  (setq result (string-to-number result)))
 	(if (null (setq mp (nthcdr (1- result) vm-message-list)))
 	    (error "No such message.")))
       (car mp))))
@@ -281,82 +318,133 @@ vm-included-text-prefix is prepended to every yanked line."
 	     (vm-number-of message)))
   (vm-display nil nil '(vm-yank-message) '(vm-yank-message composing-message))
   (setq message (vm-real-message-of message))
-  (let ((b (current-buffer)) (start (point)) end)
+  (let ((b (current-buffer)) (start (point)) end insert-start)
     (save-restriction
       (widen)
       (save-excursion
-	(if (vectorp (vm-mm-layout message))
-	    (let* ((o (vm-mm-layout message))
-		   (type (car (vm-mm-layout-type o)))
-		   parts)
-	      (vm-insert-region-from-buffer (vm-buffer-of message)
-					    (vm-headers-of message)
-					    (vm-text-of message))
-	      ;; decode MIME encoded words so supercite and other
-	      ;; mail-citation-hook denizens won't have to eat 'em.
-	      (if vm-display-using-mime
-		  (save-restriction
-		    (narrow-to-region start (point))
-		    (vm-decode-mime-encoded-words)))
-	      (cond ((vm-mime-types-match "multipart" type)
-		     (setq parts (copy-sequence (vm-mm-layout-parts o))))
-		    (t (setq parts (list o))))
-	      (while parts
-		(cond ((vm-mime-text-type-layout-p (car parts))
-		       (if (cond ((vm-mime-types-match
-				   "text/enriched"
-				   (car (vm-mm-layout-type (car parts))))
-				  (vm-mime-display-internal-text/enriched
-				   (car parts)))
+        (if vm-reply-include-presentation
+            (let ((text
+                   (save-excursion
+		     (vm-select-folder-buffer)
+		     ;; ensure the current message is presented 
+		     (vm-show-current-message)
+		     (vm-select-folder-buffer)
+		     (if vm-presentation-buffer
+			 (set-buffer vm-presentation-buffer))
+                     (vm-buffer-substring-no-properties (point-min) (point-max)))))
+              (insert text)
+	      (setq end (point-marker)))
+          (if (vectorp (vm-mm-layout message))
+              (let* ((o (vm-mm-layout message))
+                     layout new-layout
+                     (type (car (vm-mm-layout-type o)))
+                     (alternatives 0)
+                     (parts (list o)))
+                (vm-insert-region-from-buffer (vm-buffer-of message)
+                                              (vm-headers-of message)
+                                              (vm-text-of message))
+                (while parts
+                  (setq layout (car parts))
+                  (cond ((vm-mime-text-type-layout-p layout)
+                         (if (cond ((vm-mime-types-match
+                                     "text/enriched"
+                                     (car (vm-mm-layout-type layout)))
+                                    (vm-mime-display-internal-text/enriched
+                                     layout))
+                                   ((vm-mime-types-match
+                                     "message/rfc822"
+                                     (car (vm-mm-layout-type layout)))
+                                    (vm-mime-display-internal-message/rfc822
+                                     layout))
 ;; no text/html for now
 ;;				 ((vm-mime-types-match
 ;;				   "text/html"
-;;				   (car (vm-mm-layout-type (car parts))))
+;;				   (car (vm-mm-layout-type layout)))
 ;;				  (vm-mime-display-internal-text/html
-;;				   (car parts)))
-				 ((vm-mime-display-internal-text/plain
-				   (car parts) t)))
-			   nil
-			 ;; charset problems probably
-			 ;; just dump the raw bits
-			 (vm-mime-insert-mime-body (car parts))
-			 (vm-mime-transfer-decode-region (car parts)
-							 start (point)))
-		       (setq parts (cdr parts)))
-		      ((vm-mime-composite-type-p
-			(car (vm-mm-layout-type (car parts))))
-		       (setq parts (nconc (copy-sequence
-					   (vm-mm-layout-parts
-					    (car parts)))
-					  (cdr parts))))
-		      (t (setq parts (cdr parts)))))
-	      (setq end (point-marker)))
-	  (set-buffer (vm-buffer-of message))
-	  (save-restriction
-	    (widen)
-	    ;; decode MIME encoded words so supercite and other
-	    ;; mail-citation-hook denizens won't have to eat 'em.
-	    (append-to-buffer b (vm-headers-of message)
-			      (vm-text-end-of message))
-	    (set-buffer b)
-	    (setq end (point-marker))
-	    (if vm-display-using-mime
-		(progn
-		  (narrow-to-region start end)
-		  (vm-decode-mime-encoded-words))))))
+;;				   layout))
+                                   ((member (downcase (car (vm-mm-layout-type
+                                                            layout)))
+                                            vm-included-mime-types-list)
+                                    (vm-mime-display-internal-text/plain
+                                     layout t))
+                                   ;; convert the layout if possible
+                                   ((and (not (vm-mm-layout-is-converted layout))
+                                         (vm-mime-can-convert (car (vm-mm-layout-type
+                                                                    layout)))
+                                         (setq new-layout
+                                               (vm-mime-convert-undisplayable-layout
+                                                layout)))
+                                    (vm-decode-mime-layout new-layout)))
+                             ;; we have found a part to insert, thus skip the
+                             ;; remaining alternatives  
+                             (while (> alternatives 1)
+                               (setq parts (cdr parts)
+                                     alternatives (1- alternatives)))
+			 
+                           (if (not (member (downcase (car (vm-mm-layout-type
+                                                            layout)))
+                                            vm-included-mime-types-list))
+                               nil
+                             ;; charset problems probably
+                             ;; just dump the raw bits
+                             (setq insert-start (point))
+                             (vm-mime-insert-mime-body layout)
+                             (vm-mime-transfer-decode-region layout
+                                                             insert-start
+                                                             (point))))
+                         (setq alternatives (1- alternatives))
+                         (setq parts (cdr parts)))
+                        ;; burst composite types 
+                        ((vm-mime-composite-type-p
+                          (car (vm-mm-layout-type layout)))
+                         (setq alternatives (length (vm-mm-layout-parts (car parts))))
+                         (setq parts (nconc (copy-sequence
+                                             (vm-mm-layout-parts
+                                              (car parts)))
+                                            (cdr parts))))
+                        ;; skip non-text parts 
+                        (t
+                         (setq alternatives (1- alternatives))
+                         (setq parts (cdr parts)))))
+                (setq end (point-marker)))
+            (set-buffer (vm-buffer-of message))
+            (save-restriction
+              (widen)
+              ;; decode MIME encoded words so supercite and other
+              ;; mail-citation-hook denizens won't have to eat 'em.
+              (append-to-buffer b (vm-headers-of message)
+                                (vm-text-end-of message))
+              (set-buffer b)
+              (setq end (point-marker))
+              (if vm-display-using-mime
+                  (progn
+                    (narrow-to-region start end)
+                    (vm-decode-mime-encoded-words)))))))
       ;; get rid of read-only text properties on the text, as
       ;; they will only cause trouble.
       (let ((inhibit-read-only t))
-	(remove-text-properties (point-min) (point-max) '(read-only nil)
+	(remove-text-properties (point-min) (point-max)
+				'(read-only nil	invisible nil)
 				(current-buffer)))
-      (push-mark end)
-      (cond (mail-citation-hook (run-hooks 'mail-citation-hook))
-	    (mail-yank-hooks (run-hooks 'mail-yank-hooks))
-	    (t (vm-mail-yank-default message))))))
+      
+      ;; decode MIME encoded words so supercite and other
+      ;; mail-citation-hook denizens won't have to eat 'em.
+      (if vm-display-using-mime
+          (save-restriction
+            (narrow-to-region start end)
+            (vm-decode-mime-encoded-words))))
+      
+    (push-mark end)
+    (cond (mail-citation-hook (run-hooks 'mail-citation-hook))
+          (mail-yank-hooks (run-hooks 'mail-yank-hooks))
+          (t (vm-mail-yank-default message)))))
 
+;;;###autoload
 (defun vm-mail-send-and-exit (&rest ignored)
   "Send message and maybe delete the composition buffer.
-The value of `vm-keep-sent-mesages' determines whether the composition buffer is deleted.  If the composition is a reply to a message ina currenttly visited folder, that message is marked as having been rpelied to."
+The value of `vm-keep-sent-mesages' determines whether the composition buffer
+is deleted.  If the composition is a reply to a message in a currently visited
+folder, that message is marked as having been replied to."  
   (interactive "P")
   (vm-check-for-killed-folder)
   (if (and (boundp 'mail-alias-file)
@@ -435,18 +523,18 @@ The value of `vm-keep-sent-mesages' determines whether the composition buffer is
 	  (goto-char (point-min))
 	  (insert (format "%sDate: " (if resent "Resent-" ""))
 		  (capitalize
-		   (car (nth (string-to-int (format-time-string "%w" time))
+		   (car (nth (string-to-number (format-time-string "%w" time))
 			     vm-weekday-alist)))
 		  ", "
 		  ;; %e generated " 2".  Go from string to int
 		  ;; to string to get rid of the blank.
 		  (int-to-string
-		   (string-to-int
+		   (string-to-number
 		    (format-time-string "%e" time)))
 		  " "
 		  (capitalize
 		   (car (nth
-			 (1- (string-to-int (format-time-string "%m" time)))
+			 (1- (string-to-number (format-time-string "%m" time)))
 			 vm-month-alist)))
 		  (format-time-string " %Y %H:%M:%S" time)
 		  (format " %s%02d%02d"
@@ -496,6 +584,7 @@ The value of `vm-keep-sent-mesages' determines whether the composition buffer is
 
 (defvar coding-system-for-write)
 
+;;;###autoload
 (defun vm-mail-send ()
   "Just like mail-send except that VM flags the appropriate message(s)
 as replied to, forwarded, etc, if appropriate."
@@ -575,6 +664,7 @@ as replied to, forwarded, etc, if appropriate."
 	  (vm-keep-mail-buffer (current-buffer))))
     (vm-display nil nil '(vm-mail-send) '(vm-mail-send))))
 
+;;;###autoload
 (defun vm-mail-mode-get-header-contents (header-name-regexp)
   (let (regexp)
     (setq regexp (concat "^\\(" header-name-regexp "\\)\\|\\(^"
@@ -591,6 +681,7 @@ as replied to, forwarded, etc, if appropriate."
 	      (vm-matched-header-contents)
 	    nil ))))))
 
+;;;###autoload
 (defun vm-mail-mode-remove-header (header-name-regexp)
   (let (regexp)
     (setq regexp (concat "^\\(" header-name-regexp "\\)\\|\\(^"
@@ -672,11 +763,12 @@ as replied to, forwarded, etc, if appropriate."
 	(setq mp (cdr mp)))
       (vm-update-summary-and-mode-line))))
 
+;;;###autoload
 (defun vm-reply (count)
   "Reply to the sender of the current message.
 Numeric prefix argument N means to reply to the current message plus the
 next N-1 messages.  A negative N means reply to the current message and
-the previous N-1 messages. 
+the previous N-1 messages.
 
 If invoked on marked messages (via vm-next-command-uses-marks),
 all marked messages will be replied to.
@@ -699,6 +791,7 @@ with C-c C-v."
   (vm-error-if-folder-empty)
   (vm-do-reply nil nil count))
 
+;;;###autoload
 (defun vm-reply-include-text (count)
   "Reply to the sender (only) of the current message and include text
 from the message.  See the documentation for function vm-reply for details."
@@ -709,6 +802,7 @@ from the message.  See the documentation for function vm-reply for details."
   (vm-error-if-folder-empty)
   (vm-do-reply nil t count))
 
+;;;###autoload
 (defun vm-followup (count)
   "Reply to all recipients of the current message.
 See the documentation for the function vm-reply for details."
@@ -719,6 +813,7 @@ See the documentation for the function vm-reply for details."
   (vm-error-if-folder-empty)
   (vm-do-reply t nil count))
 
+;;;###autoload
 (defun vm-followup-include-text (count)
   "Reply to all recipients of the current message and include text from
 the message.  See the documentation for the function vm-reply for details."
@@ -729,6 +824,7 @@ the message.  See the documentation for the function vm-reply for details."
   (vm-error-if-folder-empty)
   (vm-do-reply t t count))
 
+;;;###autoload
 (defun vm-forward-message-all-headers ()
   "Like vm-forward-message but always forwards all the headers."
   (interactive)
@@ -744,6 +840,7 @@ the message.  See the documentation for the function vm-reply for details."
 	(vm-mime-digest-discard-header-regexp "only-drop-this-header"))
     (vm-forward-message)))
 
+;;;###autoload
 (defun vm-forward-message ()
   "Forward the current message to one or more recipients.
 You will be placed in a Mail mode buffer as you would with a
@@ -835,6 +932,7 @@ Subject: header manually."
       (run-hooks 'vm-forward-message-hook)
       (run-hooks 'vm-mail-mode-hook))))
 
+;;;###autoload
 (defun vm-resend-bounced-message ()
   "Extract the original text from a bounced message and resend it.
 You will be placed in a Mail mode buffer with the extracted message and
@@ -899,6 +997,7 @@ you can change the recipient address before resending the message."
   (run-hooks 'vm-resend-bounced-message-hook)
   (run-hooks 'vm-mail-mode-hook))
 
+;;;###autoload
 (defun vm-resend-message ()
   "Resend the current message to someone else.
 The current message will be copied to a Mail mode buffer and you
@@ -961,6 +1060,7 @@ You may also create a Resent-Cc header."
       (run-hooks 'vm-resend-message-hook)
       (run-hooks 'vm-mail-mode-hook))))
 
+;;;###autoload
 (defun vm-send-digest (&optional prefix)
   "Send a digest of all messages in the current folder to recipients.
 The type of the digest is specified by the variable vm-digest-send-type.
@@ -989,7 +1089,15 @@ only marked messages will be put into the digest."
 	start header-end boundary)
     (save-restriction
       (widen)
-      (vm-mail-internal (format "digest from %s" (buffer-name)))
+      (vm-mail-internal
+       (format "digest from %s" (buffer-name))
+       nil
+       (and vm-forwarding-subject-format
+            (let ((vm-summary-uninteresting-senders nil))
+              (concat (vm-summary-sprintf vm-forwarding-subject-format (car mlist))
+                      (if (cdr mlist)
+                          (format " [and %d more messages]"
+                                  (length (cdr mlist))))))))
       (make-local-variable 'vm-forward-list)
       (setq vm-system-state 'forwarding
 	    vm-forward-list mlist
@@ -1031,7 +1139,14 @@ only marked messages will be put into the digest."
 	    ((equal vm-digest-send-type "rfc1153")
 	     (vm-rfc1153-encapsulate-messages
 	      mlist vm-rfc1153-digest-headers
-	      vm-rfc1153-digest-discard-header-regexp)))
+	      vm-rfc1153-digest-discard-header-regexp))
+            ((equal vm-digest-send-type nil)
+             (while mlist
+               (vm-no-frills-encapsulate-message
+                (car mlist) vm-forwarded-headers
+                vm-unforwarded-header-regexp)
+               (setq mlist (cdr mlist)))))
+
       (goto-char start)
       (setq mp mlist)
       (if miming
@@ -1067,24 +1182,28 @@ only marked messages will be put into the digest."
   (run-hooks 'vm-send-digest-hook)
   (run-hooks 'vm-mail-mode-hook))
 
+;;;###autoload
 (defun vm-send-rfc934-digest (&optional preamble)
   "Like vm-send-digest but always sends an RFC 934 digest."
   (interactive "P")
   (let ((vm-digest-send-type "rfc934"))
     (vm-send-digest preamble)))
 
+;;;###autoload
 (defun vm-send-rfc1153-digest (&optional preamble)
   "Like vm-send-digest but always sends an RFC 1153 digest."
   (interactive "P")
   (let ((vm-digest-send-type "rfc1153"))
     (vm-send-digest preamble)))
 
+;;;###autoload
 (defun vm-send-mime-digest (&optional preamble)
   "Like vm-send-digest but always sends an MIME (multipart/digest) digest."
   (interactive "P")
   (let ((vm-digest-send-type "mime"))
     (vm-send-digest preamble)))
 
+;;;###autoload
 (defun vm-continue-composing-message (&optional not-picky)
   "Find and select the most recently used mail composition buffer.
 If the selected buffer is already a Mail mode buffer then it is
@@ -1172,12 +1291,28 @@ found, the current buffer remains selected."
 (defvar mail-signature-file)
 (defvar mail-personal-alias-file)
 
+(defun vm-drop-buffer-name-chars (buffer-name)
+  "Replace chars matching `vm-drop-buffer-name-chars' by an \"_\"."
+  (let ((r vm-drop-buffer-name-chars))
+    (if (eq r t) (setq r "[^\x0-\x80]"))
+    (if (and buffer-name r)
+        (replace-in-string buffer-name r "_" t)
+      buffer-name)))
+
+;;;###autoload
 (defun vm-mail-internal
     (&optional buffer-name to subject in-reply-to cc references newsgroups)
+    "Create a message buffer and set it up according to args.
+Fills in the headers as given by the arguments.
+Binds the `vm-mail-mode-map' and hooks"
   (let ((folder-buffer nil))
     (if (memq major-mode '(vm-mode vm-virtual-mode))
 	(setq folder-buffer (current-buffer)))
-    (set-buffer (generate-new-buffer (or buffer-name "mail to ?")))
+    (setq buffer-name (if buffer-name
+                          (vm-decode-mime-encoded-words-in-string buffer-name)
+                        "mail to ?"))
+    (setq buffer-name (vm-drop-buffer-name-chars buffer-name))
+    (set-buffer (generate-new-buffer buffer-name))
     ;; FSF Emacs: try to prevent write-region (called to handle FCC) from
     ;; asking the user to choose a safe coding system.
     (if (and vm-fsfemacs-mule-p (fboundp 'set-buffer-file-coding-system))
@@ -1216,15 +1351,33 @@ found, the current buffer remains selected."
 		(build-mail-aliases)))))
     (if (stringp vm-mail-header-from)
 	(insert "From: " vm-mail-header-from "\n"))
+    (setq to (if to (vm-decode-mime-encoded-words-in-string to))
+	  subject (if subject (vm-decode-mime-encoded-words-in-string subject))
+	  cc (if cc (vm-decode-mime-encoded-words-in-string cc)))
     (insert "To: " (or to "") "\n")
     (and cc (insert "Cc: " cc "\n"))
     (insert "Subject: " (or subject "") "\n")
     (and newsgroups (insert "Newsgroups: " newsgroups "\n"))
     (and in-reply-to (insert "In-Reply-To: " in-reply-to "\n"))
     (and references (insert "References: " references "\n"))
-    (insert "X-Mailer: VM " vm-version " under "
-	    (if vm-fsfemacs-p "Emacs " "")
-	    emacs-version "\n")
+    (insert "X-Mailer: VM " (vm-version) " under ")
+    (cond ((vm-fsfemacs-p)
+	   (insert "Emacs " emacs-version))
+	  ((featurep 'xemacs)
+	   (insert
+	    (format "XEmacs %d.%d.%d" 
+		    emacs-major-version 
+		    emacs-minor-version 
+		    emacs-major-version)))
+	  ((boundp 'emacs-version)
+	   (insert emacs-version))
+	  (t
+	   (insert "Unknown Emacs")))
+    (if (functionp 'emacsw32-version)
+	(insert " [" (emacsw32-version) "]"))
+    (if (boundp 'system-configuration)
+	(insert " (" system-configuration ")"))
+    (insert "\n")
     ;; REPLYTO environmental variable support
     ;; note that in FSF Emacs v19.29 we would initialize if the
     ;; value was t.  nil is the trigger value used now.
@@ -1260,7 +1413,7 @@ found, the current buffer remains selected."
 						  mail-signature-file)
 					     "~/.signature")))
 		  (t
-		   (let ((str (eval mail-signature))) 
+		   (let ((str (eval mail-signature)))
 		     (if (stringp str)
 			 (insert str)))))
 	    (goto-char (point-min))
@@ -1321,6 +1474,7 @@ found, the current buffer remains selected."
 		 (run-with-idle-timer 1.5 t 'vm-update-composition-buffer-name))))
     (run-hooks 'mail-setup-hook)))
 
+;;;###autoload
 (defun vm-reply-other-frame (count)
   "Like vm-reply, but run in a newly created frame."
   (interactive "p")
@@ -1332,6 +1486,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-reply-include-text-other-frame (count)
   "Like vm-reply-include-text, but run in a newly created frame."
   (interactive "p")
@@ -1343,6 +1498,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-followup-other-frame (count)
   "Like vm-followup, but run in a newly created frame."
   (interactive "p")
@@ -1354,6 +1510,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-followup-include-text-other-frame (count)
   "Like vm-followup-include-text, but run in a newly created frame."
   (interactive "p")
@@ -1365,6 +1522,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-forward-message-all-headers-other-frame ()
   "Like vm-forward-message-all-headers, but run in a newly created frame."
   (interactive)
@@ -1376,6 +1534,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-forward-message-other-frame ()
   "Like vm-forward-message, but run in a newly created frame."
   (interactive)
@@ -1387,6 +1546,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-resend-message-other-frame ()
   "Like vm-resend-message, but run in a newly created frame."
   (interactive)
@@ -1398,6 +1558,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-resend-bounced-message-other-frame ()
   "Like vm-resend-bounced-message, but run in a newly created frame."
   (interactive)
@@ -1409,6 +1570,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-send-digest-other-frame (&optional prefix)
   "Like vm-send-digest, but run in a newly created frame."
   (interactive "P")
@@ -1420,6 +1582,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-send-rfc934-digest-other-frame (&optional prefix)
   "Like vm-send-rfc934-digest, but run in a newly created frame."
   (interactive "P")
@@ -1431,6 +1594,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-send-rfc1153-digest-other-frame (&optional prefix)
   "Like vm-send-rfc1153-digest, but run in a newly created frame."
   (interactive "P")
@@ -1442,6 +1606,7 @@ found, the current buffer remains selected."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-send-mime-digest-other-frame (&optional prefix)
   "Like vm-send-mime-digest, but run in a newly created frame."
   (interactive "P")
@@ -1455,6 +1620,7 @@ found, the current buffer remains selected."
 
 (defvar enriched-mode)
 
+;;;###autoload
 (defun vm-preview-composition ()
   "Show how the current composition buffer might be displayed
 in a MIME-aware mail reader.  VM copies and encodes the current
@@ -1538,8 +1704,10 @@ message."
               newbufname (format fmt newbufname ellipsis))
         (if (equal newbufname curbufname)
             nil
+          (setq newbufname (vm-drop-buffer-name-chars newbufname))
           (rename-buffer newbufname t)))))
 
+;;;###autoload
 (defun vm-mail-mode-remove-tm-hooks ()
   (remove-hook 'mail-setup-hook 'turn-on-mime-edit)
   (remove-hook 'mail-setup-hook 'mime/decode-message-header)
@@ -1548,3 +1716,5 @@ message."
   (remove-hook 'mail-send-hook 'mime-editor/maybe-translate))
 
 (provide 'vm-reply)
+
+;;; vm-reply.el ends here
